<?php
	
	// Barometer script for Bosh BMP180 sensor using i2c interface
	// Copyright (c) 2015 Diving-91 (User:diving91 https://www.jeedom.fr/forum/)
	// Trend Class adapted form User:Edualc https://www.jeedom.fr/forum/
	// 
	// Device datasheet http://ae-bst.resource.bosch.com/media/products/dokumente/bmp180/BST-BMP180-DS000-12~1.pdf
	//
	// i2c-tools have to be installed
	// sudo raspi-config -> enable I2C
	// sudo apt-get install i2c-tools
	// sudo nano /etc/modules ande add lines
	//	i2c-bcm2708 
	//	i2c-dev
	// sudo nano /etc/modprobe.d/raspi-blacklist.conf and comment following line
	//	#blacklist i2c-bcm2708
	// sudo nano /boot/config.txt add or uncomment line
	//	dtparam=i2c_arm=on
	// add user www-data in i2c group
	// sudo nano /etc/group -> i2c:x:997:pi,www-data
	// or sudo adduser www-data i2c
	// reboot
	//	sudo shutdown -r now
	// 
	
	// Class I2cBus
	// Class Bmp180
	//	Method getTemperature() 			Returns Temperature in °C
	//	Method getDeviceLevelPressure()			Returns Pressure in hPA at device altitude
	//	Method getDeviceAltitude($seaLevelPressurehPA)	Returns device altitude in meters knowing Sea Leval pressure in hPA
	//	Method getSeaLevelPressure($altitude)		Returns Sea Level Pressure in hPA knowing device Altitude in meters
	// Class Trend
	//	sources : http://www.freescale.com/files/sensors/doc/app_note/AN3914.pdf
	//	https://www.parallax.com/sites/default/files/downloads/29124-Altimeter-Application-Note-501.pdf
	//	This Class only work with Jeedom Framework (CLI mode)
	//	Method getTrend($barometerID)			Returns barometric tendancy (5 down to 0) for specified Jeedom barometer ID 
	//
	// Usage
	//	/usr/share/nginx/www/jeedom/plugins/script/core/ressources/bmp180.php t
	//	/usr/share/nginx/www/jeedom/plugins/script/core/ressources/bmp180.php dlp
	//	/usr/share/nginx/www/jeedom/plugins/script/core/ressources/bmp180.php alt=hpa
	//	/usr/share/nginx/www/jeedom/plugins/script/core/ressources/bmp180.php slp=meters
	//	/usr/share/nginx/www/jeedom/plugins/script/core/ressources/bmp180.php id=jeedomID //Returns trend
	
// Set to true or false to log some debug information
$debugMode= false;

class I2cBus {
	private $bus = 1; // i2c bus number ( 0 on first RPI generation, 1 for recent ones)
	private $i2c_address; // device address

	public function __construct ($i2c_address) {
		$this->i2c_address = $i2c_address;
	}

	public function read_register($register) {
		return trim(shell_exec('i2cget -y ' . $this->bus . ' ' . $this->i2c_address . ' ' . $register));
	}

	public function write_register($register,$value) {
		shell_exec('i2cset -y ' . $this->bus . ' ' . $this->i2c_address . ' ' . $register . ' ' . $value);
	}
	
	public function read_signed_short($msb_register) {
		$msb = intval( $this->read_register( $msb_register ), 16 );
		$lsb = intval( $this->read_register( $msb_register + 1 ), 16 );
		$val = ( $msb << 8 ) + $lsb;
		$array = unpack( 's', pack( 'v', $val ) );
		$decimal_value = $array[1];
		return $decimal_value;
	}	

	public function read_unsigned_short($msb_register) {
		$msb = intval( $this->read_register( $msb_register ), 16 );
		$lsb = intval( $this->read_register( $msb_register + 1 ), 16 );
		$val = ( $msb << 8 ) + $lsb;
		$array = unpack( 'S', pack( 'v', $val ) );
		$decimal_value = $array[1];
		return $decimal_value;
	}
	
	public function read_unsigned_long($msb_register) {
		$msb = intval( $this->read_register( $msb_register ), 16 );
		$lsb = intval( $this->read_register( $msb_register + 1 ), 16 );
		$xlsb = intval( $this->read_register( $msb_register + 2 ), 16 );
		$val = ( $msb << 16 ) + ( $lsb << 8 ) + $xlsb;
		$array = unpack( 'l', pack( 'V', $val ) );
		$decimal_value = $array[1];
		return $decimal_value;
	}
}	
	
class Bmp180 {
	const i2c_address = 0x77; // BMP180 device address 0xEE>>1
	const device_ID= 0x55; // BMP80 device ID 
	const simulation= true; // Used to reproduce & test datasheet exemple values
	// Accuracy modes
	const ultra_low_power_mode= 0;
	const standard_mode= 1;
	const high_resolution_mode= 2;
	const ultra_high_resolution_mode= 3;
	
	private $debugMode;	// 0= production mode, 1= debug mode
	private $cliMode;	// true when in CLI mode, false when in Web server mode
	private $accuracyMode; 	// one of the above value
	private $readDelay; 	// device reading delay for Pressure depends on accuracyMode
	private $i2c; // bmp180 device on i2c bus	
	private $id; //device ID should be 0x55
	// calibration data
	private $ac1;
	private $ac2;
	private $ac3;
	private $ac4;
	private $ac5;
	private $ac6;
	private $b1;
	private $b2;
	private $mb;
	private $mc;
	private $md;
	// readings values
	private $ut;		// uncompensated temperature
	private $temperature;	// true temperature
	private $up;		// uncompensated pressure
	private $pressure;	// true pressure
	
	public function __construct($debugMode,$cliMode) {
		$this->debugMode = $debugMode;
		$this->cliMode = $cliMode;

		// Defaut Accuracy Mode - You need to change it here if needed
		$this->accuracyMode = self::high_resolution_mode;
		
		$this->i2c = new I2cBus(self::i2c_address);
		if ($this->i2c->read_register(0xd0) == self::device_ID) { // Device is present and readable on i2c Bus
			$this->readCalibrationDatas();	// get calibration data
			$this->readUncompensatedTemperature(); // get calibrated temperature
			$this->readUncompensatedPressure(); // get calibrated pressure
			$this->calculateTrueValues(); // Calculate true temperature and true pressure
		}
		else {
			if ($cliMode) log::add('script','[error]','bpm180 script: Device BMP180 not found - Check I2C is tools are present');
			else echo "Device BMP180 not found - Check I2C is tools are present</br>";
		}
	}

	private function readCalibrationDatas() {
		$this->ac1 = $this->i2c->read_signed_short( 0xaa );
		$this->ac2 = $this->i2c->read_signed_short( 0xac );
		$this->ac3 = $this->i2c->read_signed_short( 0xae );
		$this->ac4 = $this->i2c->read_unsigned_short( 0xb0 );
		$this->ac5 = $this->i2c->read_unsigned_short( 0xb2 );
		$this->ac6 = $this->i2c->read_unsigned_short( 0xb4 );
		$this->b1 = $this->i2c->read_signed_short( 0xb6 );
		$this->b2 = $this->i2c->read_signed_short( 0xb8 );
		$this->mb = $this->i2c->read_signed_short( 0xba );
		$this->mc = $this->i2c->read_signed_short( 0xbc );
		$this->md = $this->i2c->read_signed_short( 0xbe );	
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'ac1: '.$this->ac1.'</br>ac2: '.$this->ac2.'</br>ac3: '.$this->ac3.'</br>ac4: '.$this->ac4.'</br>ac5: '.$this->ac5.'</br>ac6: '.$this->ac6.'</br>';
			echo 'b1: '.$this->b1.'</br>b2: '.$this->b2.'</br>mb: '.$this->mb.'</br>mc: '.$this->mc.'</br>md: '.$this->md.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: ac1: '.$this->ac1.' ac2: '.$this->ac2.' ac3: '.$this->ac3.' ac4: '.$this->ac4.' ac5: '.$this->ac5.' ac6: '.$this->ac6);
			log::add('script','[debug]','bpm180: b1: '.$this->b1.' b2: '.$this->b2.' mb: '.$this->mb.' mc: '.$this->mc.' md: '.$this->md);
		}		
	}

	private function readUncompensatedTemperature() {
		$this->i2c->write_register( 0xf4, 0x2e );
		usleep( 4500 );
		$this->ut = $this->i2c->read_unsigned_short( 0xf6 );
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'ut: '.$this->ut.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: ut: '.$this->ut);
		}
	}

	private function readUncompensatedPressure() {
		$this->i2c->write_register( 0xf4, 0x34 + ( $this->accuracyMode << 6 ) );
		switch ($this->accuracyMode) {
			case 0: $this->readDelay = 4500; break;
			case 1: $this->readDelay = 7500; break;
			case 2: $this->readDelay = 13500; break;
			case 3: $this->readDelay = 25500; break;
		}		
		usleep( $this->readDelay );
		$this->up = $this->i2c->read_unsigned_long( 0xf6 ) >> ( 8 - $this->accuracyMode );
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'up: '.$this->up.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: up: '.$this->up);
		}
	}

	private function calculateTrueValues() {				
		// calculate true temperature
		$x1 = ( $this->ut - $this->ac6 ) * $this->ac5 >> 15;
		$x2 = ( $this->mc << 11 ) / ( $x1 + $this->md );
		$b5 = $x1 + $x2;
		$this->temperature = ( $b5 + 8 ) >> 4;
		$this->temperature = $this->temperature * 0.1; // convert to celcius
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'x1: '.$x1.'</br>x2: '.$x2.'</br>b5: '.$b5.'</br>Temperature: '.$this->temperature.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: x1: '.$x1.' x2: '.$x2.' b5: '.$b5.' Temperature: '.$this->temperature);
		}
		// calculate true pressure
		$b6 = $b5 - 4000;
		$x1 = ( $this->b2 * ( $b6 * $b6 ) >> 12 ) >> 11;
		$x2 = ( $this->ac2 * $b6 ) >> 11;
		$x3 = $x1 + $x2;
		$b3 = ( ( ( $this->ac1 * 4 + $x3 ) << $this->accuracyMode ) + 2 ) / 4;
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'b6: '.$b6.'</br>x1: '.$x1.'</br>x2: '.$x2.'</br>x3: '.$x3.'</br>b3: '.$b3.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: b6: '.$b6.' x1: '.$x1.' x2: '.$x2.' x3: '.$x3.' b3: '.$b3);
		}
		$x1 = ( $this->ac3 * $b6 ) >> 13;
		$x2 = ( $this->b1 * ( ( $b6 * $b6 ) >> 12 ) ) >> 16;
		$x3 = ( ( $x1 + $x2 ) + 2 ) >> 2;
		$b4 = ( $this->ac4 * ( $x3 + 32768 ) ) >> 15;
		$b7 = ( $this->up - $b3 ) * ( 50000 >> $this->accuracyMode );
		if( $b7 < 0x80000000 )
			$this->pressure = ( $b7 * 2 ) / $b4;
		else
			$this->pressure = ( $b7 / $b4 ) * 2;
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'x1: '.$x1.'</br>x2: '.$x2.'</br>x3: '.$x3.'</br>b4: '.$b4.'</br>b7: '.$b7.'</br>p: '.$this->pressure.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: x1: '.$x1.' x2: '.$x2.' x3: '.$x3.' b4: '.$b4.' b7: '.$b7.' p: '.$this->pressure);
		}
		$x1 = ( $this->pressure >> 8 ) * ( $this->pressure >> 8 );
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'x1: '.$x1.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: x1: '.$x1);
		}
		$x1 = ( $x1 * 3038 ) >> 16;
		$x2 = ( -7357 * $this->pressure ) >> 16;
		$this->pressure = $this->pressure + ( ( $x1 + $x2 + 3791 ) >> 4 );
		if (($this->debugMode) && (!$this->cliMode)) {
			echo 'x1: '.$x1.'</br>x2: '.$x2.'</br>Pressure: '.$this->pressure.'</br>';
		}
		if (($this->debugMode) && ($this->cliMode)) {
			log::add('script','[debug]','bpm180: x1: '.$x1.' x2: '.$x2.' Pressure: '.$this->pressure);
		}
	}
	
	public function getTemperature() {return number_format($this->temperature, 2, '.', '');}
	
	public function getDeviceLevelPressure() {return number_format($this->pressure/100, 2, '.', '');}
	
	public function getDeviceAltitude($seaLevelPressurehPA) {				
		// calculate altitude from a known sea level pressure
		$x= 44330 * ( 1 - pow( ( $this->pressure / ($seaLevelPressurehPA*100) ), ( 0.1903 ) ) );
		return number_format($x, 2, '.', '');
	}

	public function getSeaLevelPressure($altitude) {				
		// calculate pressure at sea level from a known device altitude
		$x= $this->pressure / pow( ( 1 - $altitude / 44330 ), ( 5.255 ) );
		return number_format($x/100, 2, '.', '');
	}

}

class Trend {
	public function __construct () {
    	//log::add('script','[debug]','bmp180 create trend instance');
    }
	public function getTrend($barometerID) {
		// récupération du timestamp de la dernière mesure
		$histo = new scenarioExpression();
		$collectDate = $histo -> collectDate($barometerID);
		$d = new DateTime($collectDate);
		// dernière mesure barométrique
		$d0 = $d -> modify('-10 minute'); // To be sure value is present in history table
		$d0 = $d -> format('Y-m-d H:i:00');
		$v0 = $histo->lastBetween($barometerID, $d0, $d0);
      	// calcul du timestamp - 3h
		$d3 = $d -> modify('-3 hour');
		$d3 = $d -> format('Y-m-d H:i:00');
		$v3 = $histo->lastBetween($barometerID, $d3, $d3);
		// calcul du timestamp - 6h
		$d6 = $d -> modify('-6 hour');
		$d6 = $d -> format('Y-m-d H:i:00');
		$v6 = $histo->lastBetween($barometerID, $d6, $d6);
		//log::add('script','[debug]','bmp180 tendance: d0='.$d0.' v0='.$v0.' d3='.$d3.' v3='.$v3.' d6='.$d6.' v6='.$v6);
		// compute trend
		$trend3h = ($v0 - $v3) / 3.0;
		$trend6h = ($v0 - $v6) / 6.0;
		$trend = number_format((7.0 * $trend3h + 3 * $trend6h) / 10.0 , 1);
		//log::add('script','[debug]','bmp180 tendance: $trend3h='.$trend3h.' $trend6h='.$trend6h.' $trend='.$trend);
		//below line is to debug by running the script in cli mode outside of jeedom only (work only if H:i:00 is effectively in history table)
      	//echo $collectDate.' '.$d0.' '.$v0.' '.$d3.' '.$v3.' '.$d6.' '.$v6.' '.$trend3h.' '.$trend6h.' '.$trend.' ';
		switch ($trend) {
			case ($trend > 2.5): return 5; break;
			case ($trend > 0.5): return 4; break;
			case ($trend > 0.0): return 3; break;
			case ($trend > -0.5): return 2; break;
			case ($trend > -2.5): return 1; break;
			case ($trend <= -2.5): return 0; break;
		}

		//5 Quickly rising High Pressure System, not stable
		// 'Forte embellie, instable';
		//4 Slowly rising High Pressure System, stable good weather
		// 'Amélioration, beau temps durable';
		//3  Stable weather condition
		// 'Lente amélioration, temps stable';  
		//2 Stable weather condition
		// 'Lente dégradation, temps stable';
		//1 Slowly falling Low Pressure System, stable rainy weather
		// 'Dégradation, mauvais temps durable';
		//0 Quickly falling Low Pressure, Thunderstorm, not stable
		// 'Forte dégradation, instable';
	}
}

if (php_sapi_name() == 'cli') { // script runs from a cli - assumes this is a jeedom script
	$JeedomMode = true;
	require_once dirname(__FILE__) . '/../../../../core/php/core.inc.php';
	if (isset($argv)) $query=$argv[1]; else $query='';
	if ($debugMode) {log::add('script','[debug]','bmp180 script: '.$query);}

	$query = explode('=',$query); // parameters are passed with an = after the cmd name
	if (isset($query[0])) $cmd = $query[0]; else $cmd=null;	
	if (isset($query[1])) $param = $query[1]; else $param=null;

	$barometer = new Bmp180($debugMode,false);
	if ($cmd=='t') echo $barometer->getTemperature();
	if ($cmd=='dlp') echo $barometer->getDeviceLevelPressure();
	if ($cmd=='alt') {
		if ($param !=null) echo $barometer->getDeviceAltitude($param);
		else log::add('script','[error]','bmp180 script: getDeviceAltitude is Missing Sea Level Pressure argument');
	}
	if ($cmd=='slp') {
		if ($param !=null) echo $barometer->getSeaLevelPressure($param);
		else log::add('script','[error]','bmp180 script: getSeaLevelPressure is Missing altitude argument');
	}
	if ($cmd=='id') {
		if ($param !=null) {
			$trend = new Trend(); 
			echo $trend->getTrend($param);
		}
		else log::add('script','[error]','bmp180 script: getTrend is Missing barometer ID argument');
	}
}
else {
	$barometer = new Bmp180($debugMode,false);
	echo 'T= '.$barometer->getTemperature().'</br>';//return in °C
	echo 'DLP= '.$barometer->getDeviceLevelPressure().'</br>'; //return in hPA
	echo 'ALT= '.$barometer->getDeviceAltitude(1020.80).'</br>'; //return in hPA
	echo 'SLP= '.$barometer->getSeaLevelPressure(93.5).'</br>'; //return in m
}
?>